<?php

/**
 * Fonctions utiles au modèle album
 *
 * Contient principalement des balises destinées au modèle standard
 *
 * @plugin     Albums
 * @copyright  2014
 * @author     Tetue, tcharlss
 * @licence    GNU/GPL
 * @package    SPIP\Albums\Fonctions
 */

// Sécurité
if (!defined('_ECRIRE_INC_VERSION')) {
	return;
}


/**
 * Compile la balise `#ALBUM_DOC` qui retourne le code HTML
 * d'un modèle de document ajusté pour les albums.
 *
 * Utilise en général un modèle standard `#DOC` auquel on ajoute des attributs
 * et applique des traitements d'images.
 *
 * Le choix du modèle et les traitements dépendent de la disposition de l'album et du contexte.
 *
 * S'utilise uniquement dans une boucle documents ou documents_liens.
 *
 * @uses calculer_album_doc()
 * @balise
 * @example
 *     ```
 *     #ALBUM_DOC
 *     ```
 *
 * @param object $p
 *     Pile au niveau de la balise
 * @return object
 *     Pile complétée par le code à générer
 */
function balise_ALBUM_DOC_dist($p) {

	$etoile    = $p->etoile;
	// La boucle document
	$index     = index_boucle_mere($p); // <BOUCLE_truc> → _truc
	$boucle    = $p->boucles[$index] ?? null;
	$type      = $p->boucles[$index]->type_requete ?? null; // documents_liens

	// Erreur squelette si hors boucle
	if (
		!$boucle
		or !in_array($type, ['documents', 'documents_liens'])
	) {
		erreur_squelette(
			[
				'zbug_champ_hors_boucle',
				['champ' => zbug_presenter_champ($p)]
			],
			$p
		);
	}

	// Infos sur le document
	$_document = "[
		'id_document' => " . champ_sql('id_document', $p) . ",
		'media'       => " . champ_sql('media', $p) . ",
		'titre'       => " . champ_sql('titre', $p) . ",
		'descriptif'  => " . champ_sql('descriptif', $p) . ",
		'credits'     => " . champ_sql('credits', $p) . ',
	]';

	// Contexte
	// Les params passés au modèle sont parfois dans une clé `args`, on met à plat
	$_contexte = "array_merge(\$Pile[0] ?? [], \$Pile[0]['args'] ?? [])";
	$_contexte = "nettoyer_contexte_modele_album($_contexte)";

	$p->code = "calculer_album_doc($_document, $_contexte)";
	$p->interdire_scripts = false;
	return $p;
}


/**
 * Fonction privée pour calculer le HTML de la balise #ALBUM_DOC
 *
 * @uses calculer_album_params()
 * @uses calculer_id_modele_album()
 *
 * @param array $document
 *      Tableau d'infos sur le document
 *      array<int id_document, string media, int largeur, int hauteur>
 * @param array $contexte
 *      Contexte d'appel du modèle, sans les trucs par défaut inutiles (date, date_redac, etc.)
 * @return string
 */
function calculer_album_doc(array $document, array $contexte): string {

	$html = '';
	$id_document = (int) ($document['id_document'] ?? 0);
	$media = $document['media'] ?? null;
	$titre = trim($document['titre'] ?? null);
	$descriptif = trim($document['descriptif'] ?? null);
	$credits = trim($document['credits'] ?? null);

	// Identifiant unique : id_album ou autre
	$uniqid = calculer_id_modele_album($contexte);

	// Choix du modèle, et valeurs pour les traitements à appliquer aux images
	$params         = calculer_album_params($contexte);
	$disposition    = $params['disposition'];
	$modele_doc     = $params['modele_doc'];
	$largeur        = $params['largeur'];
	$hauteur        = $params['hauteur'];
	$recadrer_ratio = $params['recadrer_ratio'];
	$recadrer_sens  = $params['recadrer_sens'];
	$images         = $params['images'];

	// On récupère le modèle en forçant l'ajout d'un lien.
	$contexte_modele = [
		'id_document' => $id_document,
		'lien' => generer_url_entite($id_document, 'document'),
	];
	$html = recuperer_fond("modeles/$modele_doc", $contexte_modele);

	// Ajouter une classe au conteneur
	$classe_caption = (($titre or $descriptif or $credits) ? 'captioned' : 'captionless');
	$classe = "album__doc album__doc_$media album__doc_$modele_doc album__doc_$classe_caption";
	$html = ajouter_class($html, $classe);

	// Mediabox : activer si c'est une image
	if (
		$media === 'image'
		and $chercher = 'class="'
		and ($pos_class = strpos($html, $chercher)) !== false
	) {
		$class_box = 'mediabox ';
		$html = substr_replace($html, $class_box, $pos_class + strlen($chercher), 0);
		// retirer la class hasbox (d'où elle sort ???)
		$html = str_replace('hasbox', '', $html);
	}

	// Mediabox : identifiant de galerie
	if (
		($pos_href = strpos($html, 'href=')) !== false
	) {
		// $attribut_rel = "rel=\"modele_album_$uniqid\" ";
		$attribut_rel = "data-box-group=\"modele_album_$uniqid\" ";
		$html = substr_replace($html, $attribut_rel, $pos_href, 0);
	}

	// Images : recadrer et redimensionner
	// NB : le modèle standard de doc permet déjà de redimensionner, mais il faut recadrer *avant*
	if ($media === 'image') {
		include_spip('inc/filtres');
		if ($recadrer_ratio) {
			$html = filtrer('image_recadre', $html, $recadrer_ratio, 'focus', $recadrer_sens);
		}
		if ($largeur or $hauteur) {
			$html = filtrer('image_reduire', $html, $largeur, $hauteur);
		}
	}

	return $html;
}


/**
 * Compile la balise `#ALBUM_PARAMS` qui retourne
 * soit un ensemble de valeurs utiles au modèle album, soit une seule.
 *
 * @uses calculer_album_params()
 * @balise
 * @example
 *     ```
 *     ALBUM_PARAMS
 *     ALBUM_PARAMS{disposition}
 *     ```
 *
 * @param object $p
 *     Pile au niveau de la balise
 * @return object
 *     Pile complétée par le code à générer
 */
function balise_ALBUM_PARAMS_dist($p) {
	// Les params passés au modèle sont parfois dans une clé `args`, on met à plat
	$_contexte = "array_merge(\$Pile[0] ?? [], \$Pile[0]['args'] ?? [])";
	$_contexte = "nettoyer_contexte_modele_album($_contexte)";
	$_param = interprete_argument_balise(1, $p) ?: null;

	$p->code = "calculer_album_params($_contexte, $_param)";
	$p->interdire_scripts = false;

	return $p;
}


/**
 * Fonction privée pour calculer la balise `#ALBUM_PARAMS`
 *
 * En fonction du contexte, de la config et des défauts,
 * on choisit la disposition, le modèle pour les documents,
 * les valeurs pour les traitements à appliquer aux images, etc.
 *
 * Chaque disposition a ses propres traitements d'image et options par défaut.
 *
 * @uses albums_decrire_dispositions()
 *
 * @param array $contexte
 *     Environnement du modèle, sans les trucs par défaut inutiles (date, date_redac, etc.)
 * @param string|null $valeur
 *     Nom d'un paramètre pour ne retourner que celui-ci
 * @return array|string
 *     Soit un tableau de params : array<string disposition, string modele_doc, string recadrer_ratio, string recadrer_sens, int largeur, int hauteur, bool images>
 *     Soit la valeur d'un param
 */
function calculer_album_params(array $contexte = [], ?string $param = null) {

	// On ne fait qu'une seule fois le travail par modèle
	static $memory;
	$uniqid = calculer_id_modele_album($contexte);
	$hash = hash('md5', $uniqid.json_encode($contexte, JSON_THROW_ON_ERROR)); // le plus rapide en php 8.1 = xxh3
	if (isset($memory[$hash])) {
		return ($param ? ($memory[$hash][$param] ?? null) : $memory[$hash]);
	}

	include_spip('inc/config');
	$memory = [];

	// Les valeurs par défaut pour chaque disposition
	$dispositions = albums_decrire_dispositions();

	// Nb : on typecaste toutes les valeurs du set pour éviter les `null`,
	// sinon le test avec `??` est invalide. Ou alors changer la façon de tester.
	$set = [];

	// À toute fin utile
	$set['uniqid'] = $uniqid;

	// Disposition
	// Priorité : params > constante > config > défaut
	$disposition_defaut = array_keys($dispositions)[0]; // 1ère de la liste
	$disposition_param = $contexte['disposition'] ?? null;
	$disposition_constant = (defined('_ALBUMS_DISPOSITION') ? _ALBUMS_DISPOSITION : null);
	$disposition_config = lire_config('albums/disposition', null);
	$disposition = $disposition_param ?: $disposition_constant ?: $disposition_config ?: $disposition_defaut;
	$set['disposition'] = (string) $disposition;

	// Modèle de document à utiliser
	// Priorité : params > défaut
	$modele_doc_disposition = $dispositions[$disposition]['modele_doc'] ?? $dispositions['*']['modele_doc'];
	$modele_doc_param = $contexte['modele_doc'] ?? null;
	$set['modele_doc'] = (string) ($modele_doc_param ?: $modele_doc_disposition);

	// Recadrer : ratio
	// Priorité : params > params obsolètes > constante > config > défaut
	$ratio_disposition = $dispositions[$disposition]['recadrer_ratio'] ?? $dispositions['*']['recadrer_ratio'];
	$ratio_constant = (defined('_ALBUMS_RECADRER_RATIO') ? _ALBUMS_RECADRER_RATIO : null);
	$ratio_config = lire_config('albums/img_ratio', null);
	$ratio_param = $contexte['recadrer_ratio'] ?? null;
	$ratio_retro = (
		($contexte['ratio'] ?? null) ?
			$contexte['ratio'] :
			(($contexte['recadrer'] ?? null) ? '1:1' : null) // on pouvait avoir `recadrer` tout seul, sans `ratio` → valeur par défaut
	);
	$set['recadrer_ratio'] = (string) ($ratio_param ?: $ratio_retro ?: $ratio_constant ?: $ratio_config ?: $ratio_disposition);

	// Recadrer : sens
	// Priorité : params > défaut
	$recadrer_sens = $contexte['recadrer_sens'] ?? '-';
	$set['recadrer_sens'] = (string) $recadrer_sens;

	// Largeur ou hauteur pour redimensionner
	// Priorité : params > constante > constante obsolète > défaut
	// Quand la disposition indique `false` comme valeur, pas de redimensionnement possible
	$largeur_disposition = $dispositions[$disposition]['largeur'] ?? $dispositions['*']['largeur'];
	$hauteur_disposition = $dispositions[$disposition]['hauteur'] ?? $dispositions['*']['hauteur'];
	$largeur_param = $contexte['largeur'] ?? null;
	$hauteur_param = $contexte['hauteur'] ?? null;
	$largeur_constant = (defined('_ALBUMS_IMG_LARGEUR') ? _ALBUMS_IMG_LARGEUR : null) ?: (defined('_ALBUMS_TAILLE_PREVIEW') ? _ALBUMS_TAILLE_PREVIEW : null);
	$hauteur_constant = (defined('_ALBUMS_IMG_HAUTEUR') ? _ALBUMS_IMG_HAUTEUR : null) ?: (defined('_ALBUMS_TAILLE_PREVIEW') ? _ALBUMS_TAILLE_PREVIEW : null);
	$set['largeur'] = (int) ($largeur_disposition === false ? null : ($largeur_param ?: $largeur_constant ?: $largeur_disposition));
	$set['hauteur'] = (int) ($hauteur_disposition === false ? null : ($hauteur_param ?: $hauteur_constant ?: $hauteur_disposition));

	// Indique s'il s'agit d'une disposition destinée aux images
	$set['images'] = $dispositions[$disposition]['images'] ?? $dispositions['*']['images'];

	// Retourner tout ou partie
	$memory[$hash] = $set;
	$retour = ($param ? ($memory[$hash][$param] ?? null) : $memory[$hash]);

	return $retour;
}


/**
 * Nettoyer le contexte pour ne garder que les trucs en rapport avec le modèle
 *
 * Permet de calculer un hash
 *
 * @param array|null $contexte
 * @return void
 */
function nettoyer_contexte_modele_album(array $contexte = []): array {

	$exclure = [
		'args', 'date', 'date_redac', 'date_default', 'date_redac_default', 'lang', 'dir_racine', 'wysiwyg', 'espace_prive',
	];
	$contexte = array_filter($contexte, function ($k) use($exclure) {return !in_array($k, $exclure);}, ARRAY_FILTER_USE_KEY);

	return $contexte;
}

/**
 * Renvoie un identifiant unique pour le modèle
 *
 * Dépend de la façon dont sont sélectionnés les documents : soit via id_album, soit id_patate, etc.
 *
 * @param array|null $contexte
 * @return string
 *     - Si album : <id_album>
 *     - Si objet : <objet>_<id_objet>
 *     - si documents : documents_<hash>
 */
function calculer_id_modele_album(array $contexte): string {
	static $ids;
	$hash = hash('md5', json_encode($contexte, JSON_THROW_ON_ERROR)); // le plus rapide en php 8.1 = xxh3
	if (isset($ids[$hash])) {
		return $ids[$hash];
	}

	$ids = [];
	// Soit un numéro d'album
	if ($id_album = $contexte['id_album'] ?? null) {
		$id = (string) $id_album;
	// Soit un type et un numéro d'objet
	} elseif ($objet = $contexte['objet'] ?? null) {
		$id_objet = $contexte['id_objet'] ?? null;
		$id = "${objet}_${id_objet}";
	// Soit des numéros de documents
	} elseif ($id_document = $contexte['id_document'] ?? null) {
		$id_document = (is_array($id_document) ? $id_document : explode(',', $id_document));
		$id_document = hash('md5', implode('-', $id_document));
		$id = "documents_$id_document";
	} else {
		$id = uniqid();
	}

	// Ajouter le hash du contexte des fois que le même album soit affiché plusieurs fois
	// $id = "${id}_${hash}";

	$ids[$hash] = $id;

	return $id;
}


/**
 * Renvoie une classe correspondant à un ratio
 *
 * Indique le format le plus proche : carré, portrait ou paysage
 * Ainsi que le ratio entier le plus proche : 2:1, 3:1
 *
 * Pour le ratio on arrondit à l'entier le plus proche, ex. : `2.21:1` → `2:1` → `2`
 *
 * @param string $ratio
 *     Ratio sous la forme `1:1`
 * @return string
 *     `format-square`, `format-landscape_2`, `format-portrait_3`, etc.
 */
function albums_classe_ratio(string $ratio): string {
	$classe = '';
	$prefixe = 'format';

	$dimensions = explode(':', $ratio) ?: [];
	$dimensions = array_map('floatval', $dimensions);
	[$largeur, $hauteur] = $dimensions;
	if (
		$hauteur > 0
		and $largeur > 0
	) {
		[$dimension_max, $dimension_min] = [max($dimensions), min($dimensions)];
		// Format
		$tolerance_carre = 1.05; // tolérance pour les images pas 100% carrées (1 = parfaitement carrée, et au delà de - en -)
		if (($dimension_max / $dimension_min) < $tolerance_carre) {
			$format = 'square';
		} elseif ($largeur > $hauteur) {
			$format = 'landscape';
		} else {
			$format = 'portrait';
		}
		$ratio = $dimension_max / $dimension_min;
		// Indication du ratio entier le plus proche : on arrondit à l'entier au dessus
		$id = ceil($ratio); //
		// Collons tout ensemble
		$classe = "${prefixe}-${format}";
		if ($format !== 'square') {
			$classe .= " ${prefixe}-${format}_${id}";
		}
	}

	return $classe;
}

/**
 * Compile le critère `{documents_album}`
 *
 * Rustine pour forcer une jointure en LEFT avec les documents liés.
 * Des fois le compilo fait du caca : plusieurs jointures sur la même table,
 * et ça pète les conditions dans le WHERE.
 *
 * @param string $idb Identifiant de la boucle
 * @param object $boucles AST du squelette
 * @param object $crit Paramètres du critère dans cette boucle
 * @return void
 */
function critere_DOCUMENTS_documents_album_dist($idb, &$boucles, $crit) {
	$boucle = &$boucles[$idb];
	$id_table = $boucle->id_table; // documents
	$primary = $boucle->primary; // id_document

	$table_join = 'documents_liens';
	$table_sql_join = "spip_$table_join";
	$boucle->from[$table_join] = $table_sql_join;
	$boucle->from_type[$table_join] = 'LEFT';
	$boucle->join[$table_join] = ["'$id_table'","'$primary'","'$primary'"];
}